@@ -3,7 +3,9 @@
 from collections import namedtuple
 from datetime import datetime, timezone, timedelta
 from passivetotal import *
-from passivetotal.analyzer._common import AnalyzerError, is_ip
+from passivetotal._version import VERSION
+from passivetotal.api import Context
+from passivetotal.analyzer._common import AnalyzerError, AnalyzerAPIError, is_ip
 
 DEFAULT_DAYS_BACK = 90
 
@@ -49,12 +51,15 @@ def init(**kwargs):
         (ArticlesRequest, 'Articles'),
         (ProjectsRequest, 'Projects'),
         (ArtifactsRequest, 'Artifacts'),
+        (MonitorRequest, 'Monitor'),
     ]
     for c, name in api_classes:
         if 'username' in kwargs and 'api_key' in kwargs:
             api_clients[name] = c(**kwargs)
         else:
-            api_clients[name] = c.from_config()
+            api_clients[name] = c.from_config(**kwargs)
+        api_clients[name].exception_class = AnalyzerAPIError
+        api_clients[name].set_context('python','passivetotal',VERSION,'analyzer')
     config['is_ready'] = True
 
 def get_api(name):
@@ -78,8 +83,12 @@ def get_object(input, type=None):
     """Get an Analyzer object for a given input and type. If no type is specified,
     type will be autodetected based on the input.
 
+    If the input is already an Analyzer object it will be returned as-is.
+
     Returns :class:`analyzer.Hostname` or :class:`analyzer.IPAddress`.
     """
+    if isinstance(input, IPAddress) or isinstance(input, Hostname):
+        return input
     objs = {
         'IPAddress': IPAddress,
         'Hostname': Hostname
@@ -88,16 +97,43 @@ def get_object(input, type=None):
         type = 'IPAddress' if is_ip(input) else 'Hostname'
     elif type not in objs.keys():
         raise AnalyzerError('type must be IPAddress or Hostname')
-    return objs[type](input)
-        
+    return objs[type](input) 
 
-def set_date_range(days_back=DEFAULT_DAYS_BACK, start=None, end=None):
+def get_version():
+    """Get the current version of this package."""
+    return VERSION
+
+def set_context(provider, variant, version, feature=''):
+    """Define the application context for an implementation using the analyzer module.
+
+    Sets a header to be sent in API requests that is used for metrics and troubleshooting.
+    
+    :param provider: The company, partner, provider or other top-level application context.
+    :param variant: The specific app, libary subcomponent, or feature category.
+    :param version: Version of the app, feature or code setting the context.
+    :param feature: Optional sub-feature, dashboard or script name.
+    """
+    for client in api_clients.values():
+        client.set_context(provider, variant, version, feature)
+
+def set_date_range(days_back=DEFAULT_DAYS_BACK, start=None, start_date=None, end=None, end_date=None):
     """Set a range of dates for all date-bounded API queries.
+
+    By default, queries will be bounded by `analyzer.DEFAULT_DAYS_BACK`. Set `days_back` to 
+    calculate the starting and ending dates automatically relative to today's date.
+
+    Or, set the `start` and `end` params explicitly using a string in YYYY-MM-DD 00:00:00
+    format or `start_date` and `end_date` params as datetime objects. 
     
     :param days_back: Number of days back to query (optional, defaults to DEFAULT_DAYS_BACK).
-    :param start: Starting date in YYYY-MM-DD 00:00:00 format; calculated automatically when days_back is set.
-    :param end: Ending date in YYYY-MM-DD 00:00:00 format; calculated automatically when days_back is set.
+    :param start: Starting date as string in YYYY-MM-DD 00:00:00 format (optional).
+    :param start_date: Starting date as datetime object (optional).
+    :param end: Ending date as string in YYYY-MM-DD 00:00:00 format (optional).
+    :param end_date: Ending date as datetime object (optional).
     """
+    if start_date and end_date:
+        start = start_date.strftime('%Y-%m-%d %H:%m:%S')
+        end = end_date.strftime('%Y-%m-%d %H:%m:%S')
     if start and end:
         config['start_date'] = start
         config['end_date'] = end
@@ -195,9 +231,26 @@ def set_project(name_or_guid, visibility='analyst', description='', tags=None, c
     else:
         raise AnalyzerError('More than one project found; narrow the search criteria or use a unique name')
 
+def AttackSurface(id_or_name=None):
+    """Find one attack surface.
+        
+    Call with no parameters to find your Attack Surface.
+    Pass a number to load a specific Attack Surface by ID, or pass a string to load the entire
+    list of Attack Surfaces and search them by case-insensitive substring.
+    
+    Raises `AnalyzerError` if no attack surfaces are found or if more than one Attack Surface name
+    matches the parameter.
+    
+    :returns: :class:`passivetotal.illuminate.AttackSurface`
+    """
+    from passivetotal.analyzer.illuminate import AttackSurface as ASI
+    return ASI.find(id_or_name)
+
+
 
 from passivetotal.analyzer.hostname import Hostname
 from passivetotal.analyzer.ip import IPAddress
 from passivetotal.analyzer.ssl import CertificateField
 from passivetotal.analyzer.articles import AllArticles
-from passivetotal.analyzer.projects import Project, ProjectList
\ No newline at end of file
+from passivetotal.analyzer.projects import Project, ProjectList
+from passivetotal.analyzer.trackers import Tracker
\ No newline at end of file
