@@ -2,7 +2,7 @@
 
 import socket
 import tldextract
-from passivetotal.analyzer import get_api, get_config, get_object
+from passivetotal.analyzer import get_api, get_object
 from passivetotal.analyzer._common import is_ip, refang, AnalyzerError
 from passivetotal.analyzer.pdns import HasResolutions
 from passivetotal.analyzer.summary import HostnameSummary, HasSummary
@@ -12,16 +12,16 @@ from passivetotal.analyzer.hostpairs import HasHostpairs
 from passivetotal.analyzer.cookies import HasCookies
 from passivetotal.analyzer.trackers import HasTrackers
 from passivetotal.analyzer.components import HasComponents
-from passivetotal.analyzer.illuminate import HasReputation
+from passivetotal.analyzer.illuminate import HasReputation, HasIntelProfiles
 from passivetotal.analyzer.articles import HasArticles
-from passivetotal.analyzer.enrich import HasMalware
+from passivetotal.analyzer.enrich import HasMalware, HasSubdomains
 from passivetotal.analyzer.projects import IsArtifact
 
 
 
 class Hostname(HasComponents, HasCookies, HasTrackers, HasHostpairs, 
                HasReputation, HasArticles, HasResolutions, HasSummary,
-               HasMalware, IsArtifact):
+               HasMalware, HasSubdomains, IsArtifact, HasIntelProfiles):
 
     """Represents a hostname such as api.passivetotal.org.
     
@@ -50,7 +50,7 @@ class Hostname(HasComponents, HasCookies, HasTrackers, HasHostpairs,
         return self
     
     def __str__(self):
-        return self._hostname
+        return '' if self._hostname is None else self._hostname
     
     def __repr__(self):
         return "Hostname('{}')".format(self.hostname)
@@ -98,7 +98,10 @@ class Hostname(HasComponents, HasCookies, HasTrackers, HasHostpairs,
     
     def _query_dns(self):
         """Perform a DNS lookup."""
-        ip = socket.gethostbyname(self._hostname)
+        try:
+            ip = socket.gethostbyname(self._hostname)
+        except Exception:
+            raise AnalyzerError('Cannot resolve IP for hostname {}'.format(self._hostname))
         self._current_ip = get_object(ip,'IPAddress')
         return self._current_ip
     
@@ -190,5 +193,21 @@ class Hostname(HasComponents, HasCookies, HasTrackers, HasHostpairs,
         return self._api_get_whois(
             compact=False
         )
+    
+    @property
+    def is_ip(self):
+        """Whether this object is an IP. Always returns false. 
+        
+        Useful when obtaining a host from a list that may provide either an analyzer.Hostname or
+        analyzer.IPAddress object."""
+        return False
+    
+    @property
+    def is_hostname(self):
+        """Whether this object is a hostname. Always returns true.
+        
+        Useful when obtaining a host froma list that may provide either an analyzer.Hostname or
+        analyzer.IPAddress object."""
+        return True
 
 
