@@ -2,7 +2,7 @@
 
 
 from passivetotal.analyzer import get_api, get_config
-from passivetotal.analyzer._common import is_ip, AnalyzerError
+from passivetotal.analyzer._common import is_ip, refang, AnalyzerError
 from passivetotal.analyzer.whois import IPWhois
 from passivetotal.analyzer.pdns import HasResolutions
 from passivetotal.analyzer.services import Services
@@ -12,7 +12,7 @@ from passivetotal.analyzer.hostpairs import HasHostpairs
 from passivetotal.analyzer.cookies import HasCookies
 from passivetotal.analyzer.trackers import HasTrackers
 from passivetotal.analyzer.components import HasComponents
-from passivetotal.analyzer.illuminate import HasReputation
+from passivetotal.analyzer.illuminate import HasReputation, HasIntelProfiles
 from passivetotal.analyzer.articles import HasArticles
 from passivetotal.analyzer.enrich import HasMalware
 from passivetotal.analyzer.projects import IsArtifact
@@ -21,7 +21,7 @@ from passivetotal.analyzer.projects import IsArtifact
 
 class IPAddress(HasComponents, HasCookies, HasHostpairs, HasTrackers, 
                 HasReputation, HasArticles, HasResolutions, HasSummary,
-                HasMalware, IsArtifact):
+                HasMalware, IsArtifact, HasIntelProfiles):
 
     """Represents an IPv4 address such as 8.8.8.8
     
@@ -36,6 +36,7 @@ class IPAddress(HasComponents, HasCookies, HasHostpairs, HasTrackers,
 
     def __new__(cls, ip):
         """Create or find an instance for the given IP."""
+        ip = refang(ip)
         if not is_ip(ip):
             raise AnalyzerError('Invalid IP address')
         self = cls._instances.get(ip)
@@ -48,7 +49,7 @@ class IPAddress(HasComponents, HasCookies, HasHostpairs, HasTrackers,
         return self
 
     def __str__(self):
-        return self._ip
+        return '' if self._ip is None else self._ip
     
     def __repr__(self):
         return "IPAddress('{}')".format(self.ip)
@@ -86,7 +87,7 @@ class IPAddress(HasComponents, HasCookies, HasHostpairs, HasTrackers,
     def _api_get_services(self):
         """Query the services API for service and port history."""
         response = get_api('Services').get_services(query=self._ip)
-        self._services = Services(response)
+        self._services = Services(response, query=self._ip)
         return self._services
     
     def _api_get_summary(self):
@@ -114,8 +115,9 @@ class IPAddress(HasComponents, HasCookies, HasHostpairs, HasTrackers,
     
     @property
     def certificates(self):
-        """History of :class:`passivetotal.analyzer.ssl.Certificates` 
-        presented by services hosted on this IP address.
+        """History of TLS certificates presented by services hosted on this IP address.
+
+        :rtype: :class:`passivetotal.analyzer.ssl.Certificates`
         """
         if getattr(self, '_ssl_history', None) is not None:
             return self._ssl_history
@@ -123,6 +125,10 @@ class IPAddress(HasComponents, HasCookies, HasHostpairs, HasTrackers,
     
     @property
     def services(self):
+        """Record of services observed on this host.
+
+        :rtype: :class:`passivetotal.analyzer.services.Services`
+        """
         if getattr(self, '_services', None) is not None:
             return self._services
         return self._api_get_services()
@@ -136,4 +142,20 @@ class IPAddress(HasComponents, HasCookies, HasHostpairs, HasTrackers,
         """
         if getattr(self, '_whois', None) is not None:
             return self._whois
-        return self._api_get_whois()
\ No newline at end of file
+        return self._api_get_whois()
+    
+    @property
+    def is_ip(self):
+        """Whether this object is an IP. Always returns true. 
+        
+        Useful when obtaining a host from a list that may provide either an analyzer.Hostname or
+        analyzer.IPAddress object."""
+        return True
+    
+    @property
+    def is_hostname(self):
+        """Whether this object is a hostname. Always returns false.
+        
+        Useful when obtaining a host froma list that may provide either an analyzer.Hostname or
+        analyzer.IPAddress object."""
+        return False
\ No newline at end of file
