@@ -1,11 +1,11 @@
 from datetime import datetime
 import pprint
-from passivetotal.analyzer._common import RecordList, Record, FirstLastSeen, AnalyzerError
+from passivetotal.analyzer._common import RecordList, Record, FirstLastSeen, AnalyzerError, ForPandas
 from passivetotal.analyzer import get_api, get_config, get_object
 
 
 
-class Certificates(RecordList):
+class Certificates(RecordList, ForPandas):
     
     """List of historical SSL certificates."""
 
@@ -15,11 +15,18 @@ class Certificates(RecordList):
     def _get_sortable_fields(self):
         return ['firstseen','lastseen', 'duration']
     
+    def _get_dict_fields(self):
+        return ['totalrecords']
+    
     def parse(self, api_response):
         self._records = []
         for result in api_response.get('results',[]):
             self._records.append(CertHistoryRecord(result))
     
+    @property
+    def totalrecords(self):
+        return len(self._records)
+
     @property
     def newest(self):
         """Most recently seen :class:`CertificateRecord`."""
@@ -73,7 +80,7 @@ class CertificateField:
         return self
     
     def __str__(self):
-        if not self._value:
+        if self._value is None:
             return ''
         if type(self._value) == list:
             return ','.join(self._value)
@@ -117,7 +124,7 @@ class CertificateField:
 
 
 
-class CertificateRecord(Record, FirstLastSeen):
+class CertificateRecord(Record, FirstLastSeen, ForPandas):
 
     """SSL Certificate record.
     
@@ -167,10 +174,22 @@ class CertificateRecord(Record, FirstLastSeen):
         return self._ip_history
     
     def _get_dict_fields(self):
-        fields = self.__class__._fields
-        fields.extend(['days_valid','expired'])
+        fields = ['str:{}'.format(f) for f in self.__class__._fields]
+        fields.extend(['days_valid','expired','sha1','str:firstseen','str:lastseen'])
         return fields
     
+    def to_dataframe(self, include_ips=False):
+        """Render this object as a Pandas DataFrame.
+
+        :param include_ips: Whether to include historical IP data in  the dataframe (optional, defaults to False, will likely trigger new API query for each record.)
+        :rtype: :class:`pandas.DataFrame`
+        """
+        pd = self._get_pandas()
+        as_d = self.as_dict
+        if include_ips:
+            as_d['ips'] = self.ips
+        return pd.DataFrame([as_d])
+
     @property
     def iphistory(self):
         """Get the direct API response for a history query on this certificates hash.
