@@ -1,19 +1,19 @@
-from passivetotal.analyzer._common import AsDictionary
+from passivetotal.analyzer._common import AsDictionary, ForPandas
 
 
 
-class Summary(AsDictionary):
+class Summary(AsDictionary, ForPandas):
 
     """Summary of available PassiveTotal data and key facts for hostnames & IPs."""
 
-    def _count_or_none(self, field):
+    def _count_or_zero(self, field):
         data_summary = self._summary.get('data_summary')
-        if not data_summary:
-            return None
+        if data_summary is None:
+            return 0
         field_summary = data_summary.get(field)
-        if not field_summary:
-            return None
-        return field_summary.get('count')
+        if field_summary is None:
+            return 0
+        return field_summary.get('count', 0)
     
     def __str__(self):
         return "{0.total} records available for {0.name}".format(self)
@@ -23,15 +23,33 @@ class Summary(AsDictionary):
 
     def _get_dict_fields(self):
         return ['resolutions','certificates','malware_hashes','projects','articles',
-                'total','netblock','os','asn','hosting_provider']
+                'total','netblock','os','asn','hosting_provider', 'link', 'links']
     
     def _get_dataset_fields(self):
         return ['resolutions','certificates','malware_hashes','projects','articles']
     
+    def to_dataframe(self, exclude_links=True):
+        """Render this object as a Pandas DataFrame.
+
+        :param exclude_links: Whether to exclude links from the dataframe (optional, defaults to True)
+        :rtype: :class:`pandas.DataFrame`
+        """
+        pd = self._get_pandas()
+        as_d = self.as_dict
+        as_d['host'] = self._summary['name']
+        cols = ['host','total','articles','certificates','malware_hashes','projects',
+                'resolutions','netblock','os','asn']
+        if exclude_links:
+            del(as_d['link'])
+            del(as_d['links'])
+        else:
+            cols.extend(['link','links'])
+        return pd.DataFrame([as_d], columns=cols)
+    
     @property
     def available(self):
         """List of datasets with at least one record."""
-        return [ field for field, count in self._get_dataset_fields() if count > 0 ]
+        return [ field for field in self._get_dataset_fields() if getattr(self, field) > 0 ]
     
     @property
     def total(self):
@@ -71,7 +89,7 @@ class Summary(AsDictionary):
     @property
     def resolutions(self):
         """Count of available pDNS historical resolutions."""
-        return self._count_or_none('resolutions')
+        return self._count_or_zero('resolutions')
     
     @property
     def pdns(self):
@@ -81,22 +99,37 @@ class Summary(AsDictionary):
     @property
     def certificates(self):
         """Count of available SSL certificate historical records."""
-        return self._count_or_none('certificates')
+        return self._count_or_zero('certificates')
     
     @property
     def malware_hashes(self):
         """Count of available malware hash records."""
-        return self._count_or_none('hashes')
+        return self._count_or_zero('hashes')
     
     @property
     def projects(self):
         """Count of PassiveTotal projects containing this IP or hostname."""
-        return self._count_or_none('projects')
+        return self._count_or_zero('projects')
     
     @property
     def articles(self):
         """Count of open-source intelligence (OSINT) articles referencing this IP or hostname."""
-        return self._count_or_none('articles')
+        return self._count_or_zero('articles')
+    
+    @property
+    def link(self):
+        """Link to the entire summary card in the UI."""
+        return self._summary['link']
+    
+    @property
+    def links(self):
+        """Dictionary of links to continue research on a dataset in the UI."""
+        summaries = self._summary['data_summary']
+        try:
+            summaries['malware_hashes'] = summaries['hashes']
+        except KeyError:
+            pass
+        return { name: summary['link'] for name, summary in summaries.items() if name != 'hashes' }
     
 
 
@@ -128,22 +161,22 @@ class HostnameSummary(Summary):
     @property
     def trackers(self):
         """Count of available trackers records for this hostname."""
-        return self._count_or_none('trackers')
+        return self._count_or_zero('trackers')
     
     @property
     def components(self):
         """Count of available web component records for this hostname."""
-        return self._count_or_none('components')
+        return self._count_or_zero('components')
     
     @property
     def hostpairs(self):
         """Count of available hostpair records for this hostname."""
-        return self._count_or_none('host_pairs')
+        return self._count_or_zero('host_pairs')
     
     @property
     def cookies(self):
         """Count of available cookies records for this hostname."""
-        return self._count_or_none('cookies')
+        return self._count_or_zero('cookies')
     
 
 
@@ -174,7 +207,7 @@ class IPSummary(Summary):
     @property
     def services(self):
         """Number of service (port) history records for this IP."""
-        return self._count_or_none('services')
+        return self._count_or_zero('services')
 
 
 
